// Copyright (C) 2020 Guido Berhoerster <guido+sievemgr@berhoerster.name>
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

package main

import (
	"fmt"
	"io"
	"net"
	"os"

	"go.guido-berhoerster.org/managesieve"
)

func init() {
	cmdPut.Flag.StringVar(&username, "u", "", "Set the username")
	cmdPut.Flag.StringVar(&passwordFilename, "P", "",
		"Set the name of the password file")
}

var cmdPut = &command{
	UsageLine: "put [options] host[:port] name [file]",
	Run:       runPut,
}

func runPut(cmd *command, args []string) error {
	var scriptName string
	var r io.Reader = os.Stdin
	var host, port string
	var err error
	switch len(args) {
	case 3: // name and filename
		scriptFile, err := os.Open(args[2])
		if err != nil {
			return fmt.Errorf("failed to open script file: %s\n",
				err)
		}
		defer scriptFile.Close()
		r = scriptFile
		fallthrough
	case 2: // only name
		host, port, err = parseHostPort(args[0])
		if err != nil {
			return err
		}

		scriptName = args[1]
	default:
		return usageError("invalid number of arguments")
	}

	script, err := readLimitedString(r, managesieve.ReadLimit)
	if err != nil {
		return fmt.Errorf("failed to read script: %s\n", err)
	}

	username, password, err := usernamePassword(host, port, username,
		passwordFilename)
	if err != nil {
		return err
	}

	c, err := dialPlainAuth(net.JoinHostPort(host, port), username,
		password)
	if err != nil {
		return err
	}
	defer c.Logout()

	if warnings, err := c.PutScript(scriptName, script); err != nil {
		return err
	} else if warnings != "" {
		fmt.Fprintln(os.Stderr, warnings)
	}

	return nil
}
